@@ -73,11 +73,11 @@ pub struct SpawnsetData {
     pub world_version: i32,
     pub game_mode: GameMode,
     pub non_loop_spawn_count: i32,
-    pub non_loop_length: Option<f32>,
-    pub loop_length: Option<f32>,
+    pub non_loop_length: Option<f64>,
+    pub loop_length: Option<f64>,
     pub hand: Option<u8>,
-    pub additional_gems: Option<i32>,
-    pub time_start: Option<f32>,
+    pub additional_gems: Option<f64>,
+    pub time_start: Option<f64>,
 }
 
 #[derive(serde::Deserialize, Debug)]
@@ -106,7 +106,7 @@ pub struct SpawnsetCustomLeaderboard {
 #[serde(rename_all = "camelCase")]
 pub struct SpawnsetCustomEntry {
     pub custom_entry_id: i32,
-    pub time: i32,
+    pub time: f64,
     pub has_replay: bool,
 }
 
@@ -133,13 +133,13 @@ pub struct Entry {
     pub rank: i32,
     pub id: i32,
     pub username: String,
-    pub time: i32,
+    pub time: f64,
     pub kills: i32,
     pub gems: i32,
-    pub death_type: i16,
+    pub death_type: u8,
     pub daggers_hit: i32,
     pub daggers_fired: i32,
-    pub time_total: i64,
+    pub time_total: f64,
     pub kills_total: i64,
     pub gems_total: i64,
     pub deaths_total: i64,
@@ -147,13 +147,3 @@ pub struct Entry {
     pub daggers_fired_total: i64,
 }
 
-impl Entry {
-    pub fn time_as_f32(&self) -> f32 {
-        (self.time as f32) / 10000.
-    }
-
-    pub fn time_total_as_f64(&self) -> f64 {
-        (self.time_total as f64) / 10000.
-    }
-}
-
